/* ***** BEGIN LICENSE BLOCK *****

 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * Contributor(s):
 *   Dimiter Dimitrov <dimiter@blue-edge.bg>
 *   Josh Powell <Josh@ki4bbo.org> 
 *   Jean-Bernard Marcon <jean-bernard.marcon@laposte.net>
 *
 * ***** END LICENSE BLOCK ***** */

// constructor for column
function bmhColumnTd() {
    this.node = document.createElement( "td" );
    this.offsetNode = document.createElement( "div" );
    this.node.appendChild( this.offsetNode );
}

bmhColumnTd.prototype = {

isEmpty: function() {
	     return this.offsetNode == this.node.firstChild;
	 },


getOffset: function() {
	       return this.offsetNode.offsetTop;
	   },

add: function( folderDiv ) {
	 this.node.insertBefore( folderDiv.node, this.offsetNode );
     }
}

// constructor for folder
function bmhFolderDiv( folder, index ) {
    this.node = document.createElement( "div" );
    this.node.setAttribute( "id", "folder" + index );
    this.node.setAttribute( "class", "folder" );
    this.type = folder.type;
    this.index = index;
    // hover div
    var hoverFolderH1 = bmhLib.getPrefFromArray( folder.ancestors );
    if( folder.ancestors.length > 1 ) {
	hoverFolderH1 = hoverFolderH1.replace(/^[^\|]*\|/, "" );
    }
    hoverFolderH1 = decodeURI( hoverFolderH1.replace( /\s*\|\s*/g, ">" ));
    var hoverDiv = this.createHoverDiv( hoverFolderH1, folder.description );
    // folder h1
    var folderH1 = document.createElement( "h1" );
    this.node.appendChild( folderH1 );
    // folder anchor
    var folderA = this.createAnchor( "javascript:void(0)" );
    folderA.setAttribute( "class", "headerLink" );
    //folderA.setAttribute( "id", "a" + index );
    var listId = "folderList" + this.index;
    folderA.setAttribute( "onclick", "void(bmhCheck('openFolder','" + listId + "'))" );
    this.createHeaderText( folder.ancestors, folderA ); 
    folderH1.appendChild( folderA );
    folderH1.appendChild( hoverDiv );
    // folder ul
    this.folderUl = document.createElement( "ul" );
    this.folderUl.setAttribute( "id", listId );
    this.node.appendChild( this.folderUl );
}


bmhFolderDiv.prototype = {

isEmptyFolder: function() {
	     return( this.type == "Folder"
		     ? this.folderUl.getElementsByTagName( "a" ).length == 0
		     : false );
	 },

isEmptyLivemark: function() {
		     return( this.type == "Livemark"
			     ? this.folderUl.getElementsByTagName( "a" ).length <= 1
			     : false );
		 },


		 // create folder header based on maximum header lines pref
createHeaderText: function( ancestors, folderA ) {
		      var headerArray = eval( ancestors.toSource() );
		      var top = headerArray.length - bmhLib.getPref( "maxHeaderLines" );
		      var i = top;
		      if( top > 1 ) {
			  // at least one ancestor won't be displayed
			  headerArray[top] = "> ".concat( headerArray[top] );
			  i--;
			  // append '>' for other ancestors that won't be displayed
			  for( ; i > 1; i-- ) {
			      headerArray[top] = ">".concat( headerArray[top] );
			  }
		      }
		      // top <= 1 -> all ancestors (except root) will be displayed
		      i = ( top <= 1 ? 1 : top );
		      // root ancestor only displayed for root folder
		      if( ancestors.length == 1 ) {
			  i = 0;
		      }
		      var folderName, headerBr, span;
		      for( ; i < headerArray.length; i++ ) {
			  folderName = document.createTextNode( this.processName( headerArray[i] ));	
			  span = document.createElement( "span" );
			  folderA.appendChild( span );
			  span.appendChild( folderName );
		      }
		  },

createAnchor: function( url ) {
		  var folderA = document.createElement( "a" );
		  folderA.setAttribute( "href", url );
		  folderA.setAttribute( "onmouseover", "showHoverInfo(this)" );
		  folderA.setAttribute( "onmouseout", "hideHoverInfo(this)" );
		  return folderA;
	      },

createHoverDiv: function( name, desc ) {
		    var hoverDiv = document.createElement( "div" ); 
		    hoverDiv.setAttribute( "class", "hoverDiv" );
		    hoverDiv.setAttribute( "style", "display:none" );
		    var hoverHeader = document.createElement( "h1" );
		    // may contain html
		    hoverHeader.innerHTML = name;
		    hoverDiv.appendChild( hoverHeader );
		    var descriptionSpan = document.createElement( "span" );
		    // may contain html
		    descriptionSpan.innerHTML = desc;
		    hoverDiv.appendChild( descriptionSpan );
		    return hoverDiv;
		},

add: function( name, url, description, icon, type ) {
	 // folder li
	 var folderLi = document.createElement( "li" );
	 this.folderUl.appendChild( folderLi );
	 // folder a
	 var folderA = this.createAnchor( url ); 
	 folderLi.appendChild( folderA );
	 var aName = this.processName( name );
	 // text a
	 var aText = document.createTextNode( aName );
	 // icons, list style icons doesn't work
	 var image;
	 if( icon != "" ) {
	     image = document.createElement( "img" );
	     image.setAttribute( "src", icon );
	     image.setAttribute( "alt", "" );
	     if( type == "Folder" ) {
		 image.setAttribute( "class", "normalIcon" );
	     }
	     else if( type == "Livemark" ) {
		 image.setAttribute( "class", "LivemarkIcon" );
	     }
	     // display depends on prefs
	     image.style.display = bmhLib.getIconDisplay( type, bmhLib.getPref( "iconDisplay" ));
	     folderA.appendChild( image );
	 }
	 folderA.appendChild( aText ); 
	 // add hover item
	 var hoverDiv = this.createHoverDiv( name, description ); 
	 folderLi.appendChild( hoverDiv );
     },

addSeparator: function() {
		  var folderLi = document.createElement( "li" );
		  folderLi.setAttribute( "class", "separator" );
		  this.folderUl.appendChild( folderLi );
	      },

processName: function( name ) {
		 var displayName = name;
		 var reg1 = /^(?:\w+:[\/\\]{1,3}(?:www\.)?)/; 
		 var reg2 = /.{2}$/;	
		 var max = bmhLib.getPref( "maxNameLength" );
		 if( displayName.length > max ) {
		     // remove protocol of url
		     displayName = displayName.replace( reg1, "" );
		     if( displayName.length > max ) {
			 displayName = displayName.substring( 0, max );
			 // show that name is longer than portrayed
			 displayName = displayName.replace( reg2, ".." );
		     }
		 }
		 return displayName;
	     }
}

function bmhFolder( node, ancestors, desc, type, httpReq ) {
    this.rdfNode = node;
    // ancestors include name of folder
    this.ancestors = ancestors;
    this.description = desc;
    this.type = type;
    this.httpReq = httpReq;
    if( httpReq ) {
	httpReq.ancestors = ancestors;
    }
}

var bmh = {

httpRequests: [],
	      folderDivs: [],
	      httpSuccessCounter: 0,
	      httpErrorCounter: 0,

	      main: function() {
		  // localizations in document
		  bmh.localize();
		  // RDF  
		  var rdf = new BmhRDF();
		  var curName = bmhLib.getProperty( rdf.bmRoot, rdf.NameArc, rdf.BMDS );
		  var curFolder = new bmhFolder( rdf.bmRoot, [curName], "", "Folder", null );
		  var folderStack = new Array( curFolder );
		  var inOrExcludedFolders = bmhLib.getArrayFromPref( "inOrExcludedFolders" );
		  var enumerator, tempFolders, curNode;
		  var curType, curURL, curDesc, curIcon, curNodeAncestors;
		  // search engines
		  bmh.createEngineMenu( rdf );
		  bmh.showHideSearchBar();
		  // add style 
		  bmh.addStyle( rdf );
		  var columns = bmhLib.getPref( "nrOfColumns" ); 
		  var columnTds = bmh.createColumns( columns );
		  var folderDiv, isExcluded, folderIndex = -1;
		  // make page
		  while( folderStack.length > 0 ) {
		      curFolder = folderStack.pop();
		      folderIndex++;
		      if( curFolder.httpReq ) {
			  curFolder.httpReq.folderIndex = folderIndex;
		      }
		      folderDiv = new bmhFolderDiv( curFolder, folderIndex );
		      rdf.RDFC.Init( rdf.BMDS, curFolder.rdfNode );
		      enumerator = rdf.RDFC.GetElements();
		      tempFolders = new Array();
		      while( enumerator.hasMoreElements() ) {
			  curNode = enumerator.getNext();
			  curType = bmhLib.getProperty( curNode, rdf.typeArc, rdf.BMDS )
			      .split( "#" )[1]; 
			  curName = bmhLib.getProperty( curNode, rdf.NameArc, rdf.BMDS );
			  curDesc = bmhLib.getProperty( curNode, rdf.DescriptionArc, rdf.BMDS );
			  if( curType == "Bookmark" ) {
			      // Bookmark
			      curURL = bmhLib.getProperty( curNode, rdf.URLArc, rdf.BMDS ); 
			      curIcon = bmhLib.getProperty( curNode, rdf.IconArc, rdf.BMDS );
			      // icon in link of bmh page is not picked up by FF 
			      if( /(^chrome)|(^about).*bookmarkshome/.test( curURL )) {
				  curIcon = document.getElementsByTagName( "link" )[0].getAttribute( "href" );
			      }
			      folderDiv.add( curName, curURL, curDesc, curIcon, curFolder.type );
			  }
			  else if( curType == "Folder" ) {
			      // Folder
			      curNodeAncestors = curFolder.ancestors.concat( [curName] );
			      tempFolders.unshift( new bmhFolder( curNode, curNodeAncestors, curDesc, curType, null ));
			  }
			  else if( curType == "Livemark" ) {
			      curNodeAncestors = curFolder.ancestors.concat( [curName] );
			      var httpRequest = new XMLHttpRequest();
			      httpRequest.feedURL = bmhLib.getProperty( curNode, rdf.FeedURLArc, rdf.BMDS );
			      bmh.httpRequests.push( httpRequest );
			      tempFolders.unshift( new bmhFolder( curNode, curNodeAncestors, curDesc, curType, httpRequest ));
			  }
			  else if( curType == "BookmarkSeparator" ) {
			      folderDiv.addSeparator();
			  }
		      }
		      isExcluded = bmhLib.isExcludedFolder( curFolder.ancestors.toString(),
			      bmhLib.getPref( "excludeIndex" ), inOrExcludedFolders );
		      if( isExcluded || folderDiv.isEmptyFolder() ) {
			  // folderIndex in id of folder should match index in bmh.folderDivs
			  folderIndex--;
		      }
		      else {
			  bmh.tessellate( columnTds, folderDiv );
			  bmh.folderDivs.push( folderDiv );
		      }
		      folderStack = folderStack.concat( tempFolders ); 
		  }
		  // remove empty columns
		  for( i = 0; i < columnTds.length; i++ ) {
		      if( columnTds[i].isEmpty() ) {
			  mainRow.removeChild( columnTds[i].node );
		      }
		  }
		  if( bmh.httpRequests.length == 0 ) {
		      // don't rebuild, just create static page
		      bmh.createStaticPage();
		  }
		  else {
		      for( i = 0; i < bmh.httpRequests.length; i++ ) {
			  bmh.httpGet( bmh.httpRequests[i] );
		      }
		  }
	      },

tessellate: function( columnTds, folderDiv ) {
		var index = bmh.smallestColumn( columnTds );
		columnTds[index].add( folderDiv );
	    },

createColumns:function( nr ) {
		  var oldRow = document.getElementsByTagName( "tr" )[0];
		  var newRow = document.createElement( "tr" );
		  oldRow.parentNode.replaceChild( newRow, oldRow ); 
		  var columnTds = new Array( nr );
		  for( var i = 0; i < columnTds.length; i++ ) {
		      columnTds[i] = new bmhColumnTd();
		      newRow.appendChild( columnTds[i].node );
		  }
		  return columnTds;
	      },

rebuild: function() {
	     var nrOfColumns = document.getElementsByTagName( "td" ).length;
	     var columnTds = bmh.createColumns( nrOfColumns );
	     for( i = 0; i < bmh.folderDivs.length; i++ ) {
		 bmh.tessellate( columnTds, bmh.folderDivs[i] ); 
	     }
	     bmh.folderDivs = null;
	 },

checkHttpRequests: function() {
		       var handledRequests = bmh.httpErrorCounter + bmh.httpSuccessCounter;
		       if(( bmh.httpRequests.length - handledRequests ) == 0 ) {
			   bmh.httpRequests = null;
			   if( bmh.httpSuccessCounter > 0 ) {
			       bmh.rebuild();
			   }
			   bmh.createStaticPage();
		       }
		   },

httpGet: function( req ) {
	     try {
		 bmh.httpRequestCounter++;
		 req.open( "GET", req.feedURL );
		 req.onload = bmh.httpLoaded;
		 req.onerror = bmh.httpError;
		 req.onreadystatechange = bmh.httpReadyStateChange;
		 req.setRequestHeader( "User-Agent", CommonFunc.USER_AGENT );
		 req.overrideMimeType( "application/xml" );
		 req.send( null );
	     }
	     catch( e ) {
		 bmh.httpErrorCounter++;
		 bmh.checkHttpRequests();
	     }
	 },

httpError: function( event ) {
	       event.target.abort();
	       bmh.httpErrorCounter++;
	       bmh.checkHttpRequests();
	   },

httpLoaded: function( event ) {
		try {
		    var req = event.target;
		    var oldFolderNode = document.getElementById( "folder" + req.folderIndex );
		    var oldAnchors = oldFolderNode.getElementsByTagName( "a" );
		    var imgHrefs = [], iconSrc;
		    for( var i = 0; i < oldAnchors.length; i++ ) {
			if( oldAnchors[i].firstChild.nodeName == "IMG" ) {
			    imgHrefs.push( oldAnchors[i].href );
			    iconSrc = oldAnchors[i].firstChild.src;
			}
		    }
		    imgHrefs.sort();
		    var feed = new Feed( req.responseXML );
		    var folder = new bmhFolder( null, req.ancestors, feed.getHeaderDesc(), "Livemark", null );
		    var folderDiv = new bmhFolderDiv( folder, req.folderIndex );
		    var title, link, content, iconIndex, icon;
		    for( i = 0; i < feed.items.length; i++ ) {
			iconIndex = bmhLib.binarySearch( feed.items[i].getLink(), imgHrefs );
			icon = ( iconIndex >= 0 ) ? iconSrc : "";
			folderDiv.add( feed.items[i].getTitle(), feed.items[i].getLink(), feed.items[i].getContent(), 
				icon, "Livemark" );
		    }
		    bmh.folderDivs[req.folderIndex] = folderDiv;
		    bmh.httpSuccessCounter++;
		}
		catch( e ) {
		    bmh.httpErrorCounter++;
		}
		bmh.checkHttpRequests();
	    },

createStaticPage: function() {
		      var bmhFile = bmhLib.getBmhFile();
		      if( bmhFile ) {
			  var saveDoc = document.documentElement.cloneNode( true );
			  // remove meta element with charset so browser can use default
			  metaCharset = saveDoc.getElementsByTagName( "meta" )[0];
			  metaCharset.parentNode.removeChild( metaCharset );
			  var bodies = saveDoc.getElementsByTagName( "body" );
			  bodies[0].setAttribute( "onload", "searchDivDisplay()" );
			  //var images = saveDoc.getElementsByTagName( "img" );
			  //images[0].src = document.getElementsByTagName( "link" )[0].getAttribute( "href" );
			  var scriptNodes = saveDoc.getElementsByTagName( "script" );
			  var lastScriptNode = scriptNodes[scriptNodes.length-1];
			  lastScriptNode.parentNode.removeChild( lastScriptNode );
			  var docStr = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"' + 
			      '"http://www.w3.org/TR/html4/strict.dtd">\n';
			  docStr += ("<html>\n" + saveDoc.innerHTML + "\n</html>");
			  try {
			      var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
				  .createInstance(Components.interfaces["nsIFileOutputStream"]);
			      stream.init( bmhFile, 0x20|0x02|0x08, 0666, 0 );
			      stream.write( docStr, docStr.length );
			      stream.close();
			      if( bmh.httpErrorCounter == 0 ) {
				  // check observer component
				  var observer = Components.classes["@jeroen/bmhBookmarksObserverService;1"]
				      .getService( Components.interfaces.nsIRDFObserver );
				  // static page creation sucessful, no empty livemarks, observer exists
				  // next reload can be static page
				  bmhLib.setPref( "dynamic", 128, false );
			      }
			  }
			  catch( e ) {}
		      }
		  },


localize: function() {
	      // insert element with text for alert statement in script of document
	      var alertSpan = document.createElement( "span" );
	      alertSpan.setAttribute( "id", "alertSpan" );
	      alertSpan.style.display = "none";
	      document.getElementById( "otherLinks" ).appendChild( alertSpan );
	      // submit button and links at bottom of page
	      var readmeLink = document.getElementById( "readmeLink" );
	      var homeLink = document.getElementById( "homeLink" );
	      var optionsLink = document.getElementById( "optionsLink" );
	      var submitInput = document.getElementById( "submitInput" );
	      // alert text
	      var alertText = document.createTextNode( bmhLib.getLocaleString( "NotAvailableString" ));
	      alertSpan.appendChild( alertText );
	      document.getElementById( "otherLinks" ).appendChild( alertSpan );
	      // submit button text
	      var submitText = document.createTextNode( bmhLib.getLocaleString( "SearchButtonString" ));
	      submitInput.value = submitText.nodeValue;
	      // readme text 
	      var readmeText = document.createTextNode( bmhLib.getLocaleString( "ReadmeString" ));
	      readmeLink.appendChild( readmeText );
	      // home text 
	      var homeText = document.createTextNode( bmhLib.getLocaleString( "HomePageString" ));
	      homeLink.appendChild( homeText );
	      // options text
	      var optionsText = document.createTextNode( bmhLib.getLocaleString( "OptionsString" ));
	      optionsLink.appendChild( optionsText );
	  },


showHideSearchBar: function() {
		       var searchForm = document.getElementById( "searchForm" );
		       if( bmhLib.getPref( "hideSearchEngines" )) {
			   searchForm.style.display = "none";
		       }
		       if( searchForm.style.display != "none" ) {
			   document.getElementById( "queryInput" ).focus();
		       }
		   },

smallestColumn: function( columns ) {
		    var index = 0;
		    for(var i = 1; i < columns.length; i++ ) {
			if( columns[i].getOffset() < columns[index].getOffset() ) {
			    index = i;
			}
		    }
		    return index;
		},

showDefaultEngine: function( index ) {
		       document.getElementById( "engineSelect" ).selectedIndex = index;
		   },

addStyle: function( rdf ) {
	      try {
		  // get styles datasource
		  var stylesDS = bmhLib.getStylesDS( rdf );
		  var addon = bmhLib.getPref( "addon" );
		  if( addon != "addon 140" ) {
		      var curNode, oldTarget, newTarget;
		      var stylesList = rdf.RDF.GetResource( rdf.stylesNS + "stylesList" );
		      rdf.RDFC.Init( stylesDS, stylesList );
		      var stylesEnum = rdf.RDFC.GetElements();
		      // add hoverDiv style to all sheets
		      while( stylesEnum.hasMoreElements() ) {
			  curNode = stylesEnum.getNext().QueryInterface( rdf.nsIRDFResource );
			  oldTarget = stylesDS.GetTarget( curNode, rdf.sheetArc, true )
			      .QueryInterface( rdf.nsIRDFLiteral );
			  newTarget = rdf.RDF.GetLiteral( addon + oldTarget.Value );
			  stylesDS.Change( curNode, rdf.sheetArc, oldTarget, newTarget );
		      }
		      bmhLib.setPref( "addon", 32, "addon 140" );
		  }
		  // get default style
		  var defaultStyle = rdf.RDF.GetResource( rdf.stylesNS + bmhLib.getPref( "styleName" ));
		  var defaultSheet = bmhLib.getProperty( defaultStyle, rdf.sheetArc, stylesDS );
		  // insert stylesheet in document 
		  var styleTag = document.getElementsByTagName( "style" )[0];
		  styleTag.innerHTML = bmhLib.formatSheet( defaultSheet );
	      }
	      catch( e ) {
		  //alert( e ); 
	      }
	  },

openSidebar: function() {
		 try {
		     var mainDoc = bmhLib.getMainWindow().document;
		     var bmhBroadcasterId = "viewBmhPrefs";
		     var bmhBroadcaster = mainDoc.getElementById( bmhBroadcasterId );
		     var sidebar = mainDoc.getElementById( "sidebar" );
		     var sidebarBox = mainDoc.getElementById( "sidebar-box" );
		     var sidebarTitle = mainDoc.getElementById( "sidebar-title" );
		     var sidebarSplitter = mainDoc.getElementById( "sidebar-splitter" );
		     var sidebarBroadcasters = mainDoc.getElementsByAttribute( "group", "sidebar" );
		     for (var i = 0; i < sidebarBroadcasters.length; i++ ) {
			 sidebarBroadcasters[i].removeAttribute( "checked" );
		     }
		     bmhBroadcaster.setAttribute( "checked", "true" );
		     if( sidebarBox.hidden ) {
			 sidebarBox.hidden = false;
			 sidebarSplitter.hidden = false;
		     }
		     var url = bmhBroadcaster.getAttribute( "sidebarurl" );
		     var title = bmhBroadcaster.getAttribute( "sidebartitle" );
		     if (!title) {
			 title = bmhBroadcaster.getAttribute( "label" );
		     }
		     sidebar.setAttribute( "src", url );
		     sidebarBox.setAttribute( "src", url );
		     sidebarBox.setAttribute( "sidebarcommand", bmhBroadcaster.id );
		     sidebarTitle.setAttribute( "value", title );
		 }
		 catch( e ) {
		 }
	     },

createEngineMenu: function( rdf ) {
		      try {
			  var engineTargets = rdf.ISDS.GetTargets( rdf.rootEngine, rdf.childArc, true );
			  var curNode, curName, curIcon, searchURL, curOption, hiddenIndex, curTextNode;
			  var engineSelect = document.getElementById( "engineSelect" );
			  var dfltEngineValue = bmhLib.getPref( "engineURL" );
			  var hiddenEngines = bmhLib.getArrayFromPref( "hiddenEngines" );
			  var dfltOption = null;
			  var newHiddenEngines = [];
			  while ( engineTargets.hasMoreElements()) {
			      curNode = engineTargets.getNext().QueryInterface( Components.interfaces.nsIRDFResource );
			      curName = bmhLib.getProperty( curNode, rdf.NameArc, rdf.ISDS );
			      curIcon = bmhLib.getProperty( curNode, rdf.IconArc, rdf.ISDS );
			      try {
				  searchURL = rdf.ISEARCHSVC.GetInternetSearchURL( curNode.Value, "bmh", 0, 0, {value:0} );
			      }
			      catch( e ) {
				  searchURL = "invalid searchurl";
			      }
			      curOption = document.createElement( "option" );
			      curOption.value = curNode.Value + "|" + curIcon + "|" + searchURL;
			      // determine default engine
			      if( curNode.Value == dfltEngineValue ) {
				  dfltOption = curOption;
			      }
			      hiddenIndex = bmhLib.binarySearch( curNode.Value, hiddenEngines );
			      // check if engine is hidden
			      if( hiddenIndex >= 0 ) {
				  curOption.style.display = "none";
				  // newHidden is hidden without removed engines
				  newHiddenEngines = newHiddenEngines.concat( hiddenEngines.splice( hiddenIndex, 1 ));
			      }
			      curTextNode = document.createTextNode( curName );
			      curOption.appendChild( curTextNode );
			      engineSelect.appendChild( curOption );
			  }
			  bmhLib.setPref( "hiddenEngines", 32, bmhLib.getPrefFromArray( newHiddenEngines )); 
			  bmhLib.selectEngine( dfltOption );
		      }
		      catch( e ) {}
		  }
}



